@@ -1,6 +1,8 @@
 ### This file gives initializations.
 
 initial.em.spmd <- function(PARAM, MU = NULL){
+  X.spmd <- get("X.spmd", envir = .GlobalEnv)
+
   if(is.null(MU)){
     ### Set semi-supervised information.
     N.spmd <- nrow(X.spmd)
@@ -12,18 +14,19 @@ initial.em.spmd <- function(PARAM, MU = NULL){
     # }
 
     ### Simple random sampling from data.
-    N.allspmds <- mpi.allgather(as.integer(unlabeled.N.spmd), type = 1,
-                                  integer(COMM.SIZE))
+    N.allspmds <- spmd.allgather.integer(as.integer(unlabeled.N.spmd),
+                                         integer(.pmclustEnv$COMM.SIZE))
 
     center.spmd <- rep(0, unlabeled.K)
-    if(COMM.RANK == 0){
-      center.spmd <- sample(1:COMM.SIZE, unlabeled.K, replace = TRUE,
-                              prob = N.allspmds / sum(N.allspmds)) - 1
+    if(.pmclustEnv$COMM.RANK == 0){
+      center.spmd <- sample(1:.pmclustEnv$COMM.SIZE, unlabeled.K,
+                            replace = TRUE,
+                            prob = N.allspmds / sum(N.allspmds)) - 1
     }
-    center.spmd <- mpi.bcast(as.integer(center.spmd), type = 1)
+    center.spmd <- spmd.bcast.integer(as.integer(center.spmd))
 
     tmp <- NULL
-    n.center.spmd <- sum(center.spmd == COMM.RANK)
+    n.center.spmd <- sum(center.spmd == .pmclustEnv$COMM.RANK)
     if(n.center.spmd > 0){
       N.pool <- 1:N.spmd
       # if(SS.clustering && length(SS.id.spmd) > 0){
@@ -33,7 +36,7 @@ initial.em.spmd <- function(PARAM, MU = NULL){
       tmp <- matrix(X.spmd[id.center.spmd,], ncol = ncol(X.spmd),
                     byrow = TRUE)
     }
-    MU <- matrix(unlist(mpi.allgather.Robj(obj = tmp)),
+    MU <- matrix(unlist(spmd.allgather.object(tmp)),
                  nrow = ncol(X.spmd), ncol = unlabeled.K)
 
     ### Combind centers from semi-supervised information if any.
@@ -44,33 +47,36 @@ initial.em.spmd <- function(PARAM, MU = NULL){
   }
 
   e.step.spmd(PARAM)
+  PARAM <- em.onestep.spmd(PARAM)
   PARAM$logL <- logL.step()
   em.update.class.spmd()
 
   PARAM
 } # End of initial.em.spmd().
 
-
 initial.RndEM.spmd <- function(PARAM){
   logL.save <- -Inf
   i.iter <- 1
 
   PARAM.org <- PARAM
   repeat{
-    PARAM <- initial.em.spmd(PARAM.org)
+    PARAM <- try(initial.em.spmd(PARAM.org))
     if(class(PARAM) == "try-error"){
-       next
+      comm.cat(PARAM, "\n", quiet = TRUE)
+      next
     }
 
     N.CLASS <- get.N.CLASS(PARAM$K)
     if(any(N.CLASS < PARAM$min.N.CLASS)){
+      comm.cat("N.CLASS: ", N.CLASS, "\n", quiet = TRUE)
       next
     }
 
-    if(CONTROL$debug > 0){
-      catmpi("Initial: ", format(Sys.time(), "%H:%M:%S"),
-             ", iter: ", i.iter, ", logL: ",
-                         sprintf("%-20.10f", PARAM$logL), "\n", sep = "")
+    if(.pmclustEnv$CONTROL$debug > 0){
+      comm.cat("Initial: ", format(Sys.time(), "%H:%M:%S"),
+               ", iter: ", i.iter, ", logL: ",
+                           sprintf("%-20.10f", PARAM$logL), "\n",
+               sep = "", quiet = TRUE)
     }
 
     if(logL.save < PARAM$logL){
@@ -80,13 +86,14 @@ initial.RndEM.spmd <- function(PARAM){
     }
 
     i.iter <- i.iter + 1
-    if(i.iter > CONTROL$RndEM.iter){
+    if(i.iter > .pmclustEnv$CONTROL$RndEM.iter){
       break
     }
   }
 
-  if(CONTROL$debug > 0){
-    catmpi("Using initial iter: ", PARAM.save$initial.i.iter, "\n", sep = "")
+  if(.pmclustEnv$CONTROL$debug > 0){
+    comm.cat("Using initial iter: ", PARAM.save$initial.i.iter, "\n",
+             sep = "", quiet = TRUE)
   }
   PARAM <- initial.em.spmd(PARAM.save, MU = PARAM.save$MU)
   PARAM
